package com.caimei.config;

import com.caimei.components.RedisService;
import com.caimei.mapper.cmMapper.ClubMapper;
import com.caimei.mapper.cmMapper.LoginMapper;
import com.caimei.model.vo.WxClubUserVo;
import com.caimei.utils.JwtUtil;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

/**
 * 把切面类加入到IOC容器中
 * 所有请求都会更新redis存token的时间
 *
 * @author : Charles
 * @date : 2020/3/17
 */
@Component
@Aspect
public class GlobalTokenAspect {

    @Value("${caimei.oldapi}")
    private String oldApi;

    private RedisService redisService;
    @Autowired
    public void setRedisService(RedisService redisService) {
        this.redisService = redisService;
    }


    @Resource
    private LoginMapper loginMapper;

    @Resource
    private ClubMapper clubMapper;

    /**
     * 定义一个切入点， 从后台controller切入
     */
    @Pointcut("execution(public * com.caimei.controller.admin..*.*(..))")
    public void adminPointCut() {
    }
    
    /**
     * 定义一个切入点， 从前端controller切入
     */
    @Pointcut("execution(public * com.caimei.controller.wechat..*.*(..))")
    public void wxPointCut() {
    }

    /**
     * 前置通知
     * 在进入方法前执行 可以对参数进行限制或者拦截
     * 通常在这边做日志存储存到数据库中
     * @param joinPoint
     * @throws Throwable
     */
    @Before("adminPointCut()")
    public void adminBefore(JoinPoint joinPoint) throws Throwable {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        String token = request.getHeader("X-Token");
        String cacheToken = null!=token ? String.valueOf(redisService.get(token)) : null;
        // Redis过期后会得到"null"值，所以需判断字符串"null"
        if (cacheToken != null && cacheToken.length() != 0 && !"null".equals(cacheToken)) {
            if (JwtUtil.isVerify(cacheToken)) {
               /* String mobileOrEmail = JwtUtil.parseTokenAud(cacheToken);*/
                int userId = JwtUtil.parseTokenUid(cacheToken);
                // 刷新token
                tokenRefresh(token, userId);
            }
        }
    }

    /**
     * 前置通知
     * 在进入方法前执行 可以对参数进行限制或者拦截
     * 通常在这边做日志存储存到数据库中
     * @param joinPoint
     * @throws Throwable
     */
    @Before("wxPointCut()")
    public void userBefore(JoinPoint joinPoint) throws Throwable {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        String token = request.getHeader("X-Token");
        String cacheToken = null!=token ? String.valueOf(redisService.get(token)) : null;
        // Redis过期后会得到"null"值，所以需判断字符串"null"
        if (cacheToken != null && cacheToken.length() != 0 && !"null".equals(cacheToken)) {
            // 验证机构用户是否可以登录
            WxClubUserVo clubUser = null;
            if (cacheToken.contains(",")) {
                // 订阅号登录用户
                String[] cacheArr = cacheToken.split(",");
                String mobile = cacheArr[0];
                Integer authUserId = Integer.parseInt(cacheArr[1]);
                clubUser = clubMapper.getWxClubUser(mobile, null, authUserId, null, null);
            } else if (cacheToken.contains("|")){
                // 服务号登录用户
                String[] cacheArr = cacheToken.split("\\|");
                String openId = cacheArr[0];
                String appId = cacheArr[1];
                clubUser = loginMapper.getClubUserByOpenId(openId, appId);
            }
            if (null != clubUser) {
                // token 续签
                redisService.set(token, cacheToken, 60L * 60 * 24);
            }

        }
    }

    /**
     * JWT Token续签：
     * 业务逻辑：登录成功后，用户在未过期时间内继续操作，续签token。
     * 登录成功后，空闲超过过期时间，返回token已失效，重新登录。
     * 实现逻辑：
     * 1.登录成功后将token存储到redis里面(这时候k、v值一样都为token)，并设置过期时间为token过期时间
     * 2.当用户请求时token值还未过期，则重新设置redis里token的过期时间。
     * 3.当用户请求时token值已过期，但redis中还在，则JWT重新生成token并覆盖v值(这时候k、v值不一样了)，然后设置redis过期时间。
     * 4.当用户请求时token值已过期，并且redis中也不存在，则用户空闲超时，返回token已失效，重新登录。
     */
    public boolean tokenRefresh(String token, Integer userID) {
        String cacheToken = String.valueOf(redisService.get(token));
        // 过期后会得到"null"值，所以需判断字符串"null"
        if (cacheToken != null && cacheToken.length() != 0 && !"null".equals(cacheToken)) {
            // 校验token有效性
            if (!JwtUtil.isVerify(cacheToken)) {
                // 生成token
                String newToken = JwtUtil.createToken(userID);
                // 将token存入redis,并设置超时时间
                redisService.set(token, newToken, JwtUtil.getExpireTime());
            } else {
                // 重新设置超时时间
                redisService.set(token, cacheToken, JwtUtil.getExpireTime());
            }
            return true;
        }
        return false;
    }
}
