package com.caimei.controller.admin.auth;

import com.alibaba.fastjson.JSONObject;
import com.caimei.annotation.CurrentUser;
import com.caimei.components.RedisService;
import com.caimei.mapper.cmMapper.SystemMapper;
import com.caimei.model.ResponseJson;
import com.caimei.model.dto.ShopInfoDto;
import com.caimei.model.po.SysUser;
import com.caimei.model.po.UserPo;
import com.caimei.model.vo.*;
import com.caimei.service.auth.ShopService;
import com.caimei.utils.JwtUtil;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 供应商API
 *
 * @author : Aslee
 * @date : 2021/5/11
 */
@Api(tags = "供应商API")
@Slf4j
@RestController
@RequiredArgsConstructor
@RequestMapping("/shop")
public class ShopApi {

    private final ShopService shopService;


    private RedisService redisService;

    @Autowired
    public void setRedisService(RedisService redisService) {
        this.redisService = redisService;
    }

    @Resource
    private SystemMapper systemMapper;

    /**
     * 供应商列表
     */
    @ApiOperation("供应商列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "listType", required = false, value = "列表类型：1供应商列表，2授权信息审核列表，3资料审核列表，4医师审核列表，5已启用供应商列表，6设备分类审核列表，7授权牌物流列表"),
            @ApiImplicitParam(name = "shopName", required = false, value = "供应商名称"),
            @ApiImplicitParam(name = "loginAccount", required = false, value = "登录账号"),
            @ApiImplicitParam(name = "shopType", required = false, value = "供应商类型：1品牌方，2代理商"),
            @ApiImplicitParam(name = "infoId", required = false, value = "品牌信息id"),
            @ApiImplicitParam(name = "mobile", required = false, value = "手机号"),
            @ApiImplicitParam(name = "linkMan", required = false, value = "联系人"),
            @ApiImplicitParam(name = "lowerAuditStatus", required = false, value = "下级审核状态,0未完成审核，1已完成审核"),
            @ApiImplicitParam(name = "sendStatus", required = false, value = "寄送状态：0未寄送，1已寄送"),
            @ApiImplicitParam(name = "pageNum", required = false, value = "第几页"),
            @ApiImplicitParam(name = "pageSize", required = false, value = "一页多少条")
    })
    @GetMapping("/list")
    public ResponseJson<PageInfo<ShopListVo>> getShopList(Integer listType, String shopName,String loginAccount, Integer shopType, Integer infoId, String mobile, String linkMan, Integer lowerAuditStatus, Integer sendStatus,
                                                          @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                                          @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) {
        return shopService.getShopList(listType, shopName, loginAccount, shopType, infoId, mobile, linkMan, lowerAuditStatus, sendStatus, pageNum, pageSize);
    }

    @GetMapping("/cm/list")
    public ResponseJson<List<CmShopVo>> getCmShopList(){
        return shopService.getCmShopList();
    }

    /**
     * 获取供应商回显数据
     *
     * @param authUserId 供应商用户id
     * @return ShopFormVo
     */
    @ApiOperation("供应商回显数据")
    @ApiImplicitParam(name = "authUserId", required = true, value = "供应商用户id")
    @GetMapping("/form/data")
    public ResponseJson<ShopFormVo> getShopFormData(@CurrentUser SysUser sysUser, Integer authUserId) {
        // 管理员/供应商公用接口，管理员调用时传authUserId,供应商调用不传
        if (null != sysUser && 1 != sysUser.getId()) {
            // 获取供应商用户id
            Integer userIdentity = sysUser.getUserIdentity();
            authUserId = 2 == userIdentity ? sysUser.getId() : 3 == userIdentity ? sysUser.getParentId() : null;
            if (null == authUserId) {
                return ResponseJson.error("供应商用户id不能为空", null);
            }
        } else if (null == authUserId) {
            return ResponseJson.error("供应商用户id不能为空", null);
        }
        return shopService.getShopFormData(authUserId);
    }

    /**
     * 添加供应商
     *
     * @param params {
     *                    authUserId         供应商用户id
     *                    shopType           供应商类型：1代理商，2品牌方
     *                    shopName           供应商名称
     *                    mobile             手机号
     *                    linkMan            联系人
     *                    shopStatus         供应商状态：0停用 1启用
     *                    logo               代理商logo
     *                    qrCodeImage        公众号二维码图片
     *                    createBy           创建人用户id
     *                    shopInfo  [
     *                                  {
     *                                      infoId                  品牌信息id
     *                                      brandName               品牌名称
     *                                      brandLogo               品牌logo
     *                                      producePlace            产地
     *                                      manufacturer            制造商
     *                                  }
     *                              ]
     *                    }
     */
    @ApiOperation("添加/编辑供应商")
    @ApiImplicitParam(name = "params", value = "authUserId:供应商用户id;", required = true)
    @PostMapping("/save")
    public ResponseJson saveShop(@RequestBody String params) {
        // 由管理员调用接口，无法通过@CurrentUser注解获取供应商用户id
        JSONObject paramsMap = JSONObject.parseObject(params);
        Integer authUserId = paramsMap.getInteger("authUserId");
        Integer cmShopId = paramsMap.getInteger("cmShopId");
        Integer shopType = paramsMap.getInteger("shopType");
        if (null == shopType) {
            return ResponseJson.error("请选择供应商类型", null);
        }
        String shopName = paramsMap.getString("shopName");
        String mobile = paramsMap.getString("mobile");
        if (StringUtils.isBlank(mobile)) {
            return ResponseJson.error("参数异常，请输入手机号");
        }
        String linkMan = paramsMap.getString("linkMan");
        if (StringUtils.isBlank(mobile)) {
            return ResponseJson.error("参数异常，请输入联系人");
        }
        Integer shopStatus = paramsMap.getInteger("shopStatus");
        if (null == shopStatus) {
            return ResponseJson.error("参数异常，请输入供应商状态");
        }
        String logo = (String) paramsMap.get("logo");
        if (StringUtils.isEmpty(logo)) {
            return ResponseJson.error("供应商logo不能为空");
        }
        String prefix = (String) paramsMap.get("prefix");
        if (StringUtils.isEmpty(prefix)) {
            return ResponseJson.error("前缀不能为空");
        }
        String qrCodeImage = paramsMap.getString("qrCodeImage");
        Integer wxAccountType = paramsMap.getInteger("wxAccountType");
        String appId = paramsMap.getString("appId");
        String appSecret = paramsMap.getString("appSecret");
        Integer createBy = paramsMap.getInteger("createBy");
        /*
            组装供应商用户数据
         */
        UserPo shop = new UserPo();
        // 供应商用户id
        shop.setAuthUserId(authUserId);
        // 关联采美供应商id
        shop.setCmShopId(cmShopId);
        // 供应商名称
        shop.setName(shopName);
        // 前缀
        shop.setPrefix(prefix);
        // 手机号
        shop.setMobile(mobile);
        // 联系人
        shop.setLinkMan(linkMan);
        // 供应商状态
        shop.setStatus(shopStatus);
        // 供应商logo
        shop.setLogo(logo);
        // 公众号二维码图片
        shop.setQrCodeImage(qrCodeImage);
        // 公众号类型
        shop.setWxAccountType(wxAccountType);
        // 公众号appId
        shop.setAppId(appId);
        // 公众号appSecret
        shop.setAppSecret(appSecret);
        // 供应商类型
        shop.setShopType(shopType);
        // 创建管理员id
        shop.setCreateBy(createBy);
        // 创建时间
        shop.setCreateTime(new Date());
        List<Map<String, Object>> shopInfoData = (List<Map<String, Object>>) paramsMap.get("shopInfo");
        if (null == shopInfoData || shopInfoData.size() <= 0) {
            return ResponseJson.error("参数异常，请输入供应商信息");
        }
        List<ShopInfoDto> shopInfoList = new ArrayList<>();
        List<String> brandNameList = new ArrayList<>();
        for (Map<String, Object> infoMap : shopInfoData) {
            Integer infoId = null;
            String infoIdStr = infoMap.get("infoId").toString();
            if (StringUtils.isNotEmpty(infoIdStr)) {
                infoId = Integer.parseInt(infoIdStr);
            }
            String brandName = (String) infoMap.get("brandName");
            String brandLogo = (String) infoMap.get("brandLogo");
            String producePlace = (String) infoMap.get("producePlace");
            String manufacturer = (String) infoMap.get("manufacturer");
            if (StringUtils.isEmpty(brandName) || StringUtils.isEmpty(brandLogo) || StringUtils.isEmpty(producePlace) || StringUtils.isEmpty(manufacturer)) {
                return ResponseJson.error("供应商信息数据异常");
            }
            if (!brandNameList.contains(brandName)) {
                brandNameList.add(brandName);
            } else {
                return ResponseJson.error("品牌名称重复");
            }
            ShopInfoDto shopInfo = new ShopInfoDto();
            shopInfo.setInfoId(infoId);
            shopInfo.setBrandName(brandName);
            shopInfo.setBrandLogo(brandLogo);
            shopInfo.setProducePlace(producePlace);
            shopInfo.setManufacturer(manufacturer);
            shopInfoList.add(shopInfo);
        }
        return shopService.saveShop(shop, shopInfoList);
    }

    /**
     * 更新供应商状态
     */
    @ApiOperation("更新供应商状态")
    @ApiImplicitParam(name = "params", value = "authUserId:供应商用户id;status:供应商状态:0停用 1启用", required = true)
    @PostMapping("/update/status")
    public ResponseJson updateShopStatus(@RequestBody Map<String,Integer> params) {
        // 由管理员调用接口，无法通过@CurrentUser注解获取供应商用户id
        Integer authUserId = params.get("authUserId");
        Integer status = params.get("status");
        return shopService.updateShopStatus(authUserId, status);
    }

    /**
     * 重置密码
     */
    @ApiOperation("重置密码")
    @ApiImplicitParam(name = "params", value = "authUserId:供应商用户id", required = true)
    @PostMapping("/reset/password")
    public ResponseJson resetShopPassword(@RequestBody Map<String,Integer> params) {
        // 由管理员调用接口，无法通过@CurrentUser注解获取供应商用户id
        Integer authUserId = params.get("authUserId");
        return shopService.resetShopPassword(authUserId);
    }

    /**
     * 品牌列表
     *
     * @return AuthVo
     */
    @ApiOperation("品牌信息列表")
    @ApiImplicitParam(name = "authUserId", value = "供应商用户id", required = false)
    @GetMapping("/info/list")
    public ResponseJson<List<ShopInfoVo>> getShopInfoList(@CurrentUser SysUser sysUser, Integer authUserId) {
        // 管理员/供应商公用接口，管理员调用时传authUserId,供应商调用不传
        if (null != sysUser && 1 != sysUser.getId()) {
            // 获取供应商用户id
            Integer userIdentity = sysUser.getUserIdentity();
            authUserId = 2 == userIdentity ? sysUser.getId() : 3 == userIdentity ? sysUser.getParentId() : null;
            if (null == authUserId) {
                return ResponseJson.error("供应商用户id不能为空", null);
            }
        } else if (null == authUserId) {
            return ResponseJson.error("供应商用户id不能为空", null);
        }
        return shopService.getShopInfoList(authUserId);
    }

    /**
     * 产地国家列表
     */
    @ApiOperation("产地国家列表")
    @GetMapping("/country/list")
    public ResponseJson<List<CountryVo>> getCountryList(){
        return shopService.getCountryList();
    }

    @ApiOperation("用户反馈列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "authParty", required = false, value = "机构名称"),
            @ApiImplicitParam(name = "mobile", required = false, value = "手机号"),
            @ApiImplicitParam(name = "handleStatus", required = false, value = "处理状态：0未处理，1已处理"),
            @ApiImplicitParam(name = "pageNum", required = false, value = "第几页"),
            @ApiImplicitParam(name = "pageSize", required = false, value = "一页多少条")
    })
    @GetMapping("/feedback/list")
    public ResponseJson<PageInfo<FeedbackVo>> getFeedbackList(@CurrentUser SysUser sysUser, String authParty, String mobile, Integer handleStatus,
                                                              @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                                              @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) {
        if (null == sysUser) {
            return ResponseJson.error("用户信息异常", null);
        }
        // 获取供应商用户id
        Integer userIdentity = sysUser.getUserIdentity();
        Integer authUserId = 2 == userIdentity ? sysUser.getId() : 3 == userIdentity ? sysUser.getParentId() : null;
        if (null == authUserId) {
            return ResponseJson.error("供应商用户id不能为空", null);
        }
        return shopService.getFeedbackList(authUserId, authParty, mobile, handleStatus, pageNum, pageSize);
    }

    @ApiOperation("用户反馈表单数据")
    @ApiImplicitParam(name = "feedbackId", required = false, value = "反馈id")
    @GetMapping("/feedback/form/data")
    public ResponseJson<FeedbackVo> getFeedbackFormData(Integer feedbackId){
        return shopService.getFeedbackFormData(feedbackId);
    }

    @ApiOperation("处理用户反馈")
    @ApiImplicitParam(name = "params", required = false, value = "feedbackId:反馈id;handleResult:处理结果")
    @PostMapping("/feedback/handle")
    public ResponseJson handleFeedback(@RequestBody String params){
        JSONObject paramsMap = JSONObject.parseObject(params);
        Integer feedbackId = paramsMap.getInteger("feedbackId");
        String handleResult = paramsMap.getString("handleResult");
        return shopService.handleFeedback(feedbackId, handleResult);
    }

    @ApiOperation("修改手机号/登录账号：验证码发送")
    @ApiImplicitParam(name = "params", value = "mobile:手机号", required = true)
    @PostMapping("/verify/code/send")
    public ResponseJson sendVerifyCode(@RequestBody String params) {
        JSONObject parseObject = JSONObject.parseObject(params);
        String mobile = parseObject.getString("mobile");
        return shopService.sendVerifyCode(mobile);
    }

    @ApiOperation("修改手机号")
    @ApiImplicitParam(name = "params", value = "oldMobile:旧手机号;verifyCode:验证码;newMobile:新手机号;", required = true)
    @PostMapping("/mobile/change")
    public ResponseJson changeMobile(@CurrentUser SysUser sysUser, @RequestBody String params) {
        if (null == sysUser) {
            return ResponseJson.error("用户信息异常", null);
        }
        // 获取供应商用户id
        Integer userIdentity = sysUser.getUserIdentity();
        Integer authUserId = 2 == userIdentity ? sysUser.getId() : 3 == userIdentity ? sysUser.getParentId() : null;
        if (null == authUserId) {
            return ResponseJson.error("供应商用户id不能为空", null);
        }
        JSONObject parseObject = JSONObject.parseObject(params);
        String verifyCode = parseObject.getString("verifyCode");
        String oldMobile = parseObject.getString("oldMobile");
        String newMobile = parseObject.getString("newMobile");
        return shopService.changeMobile(authUserId, oldMobile, verifyCode, newMobile);
    }

    @ApiOperation("修改联系人")
    @ApiImplicitParam(name = "params", value = "linkMan:联系人", required = true)
    @PostMapping("/link/man/change")
    public ResponseJson changeLinkMan(@CurrentUser SysUser sysUser, @RequestBody String params) {
        if (null == sysUser) {
            return ResponseJson.error("用户信息异常", null);
        }
        // 获取供应商用户id
        Integer userIdentity = sysUser.getUserIdentity();
        Integer authUserId = 2 == userIdentity ? sysUser.getId() : 3 == userIdentity ? sysUser.getParentId() : null;
        if (null == authUserId) {
            return ResponseJson.error("供应商用户id不能为空", null);
        }
        JSONObject parseObject = JSONObject.parseObject(params);
        String linkMan = parseObject.getString("linkMan");
        return shopService.changeLinkMan(authUserId, linkMan);
    }

    @ApiOperation("修改公司logo")
    @ApiImplicitParam(name = "params", value = "logo:公司logo", required = true)
    @PostMapping("/logo/change")
    public ResponseJson changeLogo(@CurrentUser SysUser sysUser, @RequestBody String params) {
        if (null == sysUser) {
            return ResponseJson.error("用户信息异常", null);
        }
        // 获取供应商用户id
        Integer userIdentity = sysUser.getUserIdentity();
        Integer authUserId = 2 == userIdentity ? sysUser.getId() : 3 == userIdentity ? sysUser.getParentId() : null;
        if (null == authUserId) {
            return ResponseJson.error("供应商用户id不能为空", null);
        }
        JSONObject parseObject = JSONObject.parseObject(params);
        String logo = parseObject.getString("logo");
        return shopService.changeLogo(authUserId, logo);
    }

    @ApiOperation("登录账号绑定")
    @ApiImplicitParam(name = "params", value = "mobile:手机号;verifyCode:验证码;loginAccount:登录账号;", required = true)
    @PostMapping("/account/bind")
    public ResponseJson bindLoginAccount(@CurrentUser SysUser sysUser, @RequestBody String params) {
        if (null == sysUser) {
            return ResponseJson.error("用户信息异常", null);
        }
        // 获取供应商用户id
        Integer userIdentity = sysUser.getUserIdentity();
        Integer authUserId = 2 == userIdentity ? sysUser.getId() : 3 == userIdentity ? sysUser.getParentId() : null;
        if (null == authUserId) {
            return ResponseJson.error("供应商用户id不能为空", null);
        }
        JSONObject parseObject = JSONObject.parseObject(params);
        String verifyCode = parseObject.getString("verifyCode");
        String mobile = parseObject.getString("mobile");
        String loginAccount = parseObject.getString("loginAccount");
        return shopService.bindLoginAccount(authUserId, mobile, verifyCode, loginAccount);
    }

    @ApiOperation("保存供应商品牌信息")
    @ApiImplicitParam(name = "shopInfo", value = "品牌信息", required = true)
    @PostMapping("/info/save")
    public ResponseJson saveShopInfo(@CurrentUser SysUser sysUser, @RequestBody ShopInfoDto shopInfo) {
        if (null == sysUser) {
            return ResponseJson.error("用户信息异常", null);
        }
        // 获取供应商用户id
        Integer userIdentity = sysUser.getUserIdentity();
        Integer authUserId = 2 == userIdentity ? sysUser.getId() : 3 == userIdentity ? sysUser.getParentId() : null;
        if (null == authUserId) {
            return ResponseJson.error("供应商用户id不能为空", null);
        }
        return shopService.saveShopInfo(authUserId, shopInfo);
    }

    @ApiOperation("删除供应商品牌信息")
    @ApiImplicitParam(name = "params", value = "infoId:品牌信息id", required = true)
    @PostMapping("/info/delete")
    public ResponseJson saveShopInfo(@CurrentUser SysUser sysUser, @RequestBody String params) {
        if (null == sysUser) {
            return ResponseJson.error("用户信息异常", null);
        }
        // 获取供应商用户id
        Integer userIdentity = sysUser.getUserIdentity();
        Integer authUserId = 2 == userIdentity ? sysUser.getId() : 3 == userIdentity ? sysUser.getParentId() : null;
        if (null == authUserId) {
            return ResponseJson.error("供应商用户id不能为空", null);
        }
        JSONObject parseObject = JSONObject.parseObject(params);
        Integer infoId = parseObject.getInteger("infoId");
        return shopService.deleteShopInfo(authUserId, infoId);
    }

    @ApiOperation("一键更新机构授权牌")
    @PostMapping("/authImage/update/all")
    public ResponseJson updateAllAuthImage(@CurrentUser SysUser sysUser){
        if (null == sysUser) {
            return ResponseJson.error("用户信息异常", null);
        }
        // 获取供应商用户id
        Integer userIdentity = sysUser.getUserIdentity();
        Integer authUserId = 2 == userIdentity ? sysUser.getId() : 3 == userIdentity ? sysUser.getParentId() : null;
        if (null == authUserId) {
            return ResponseJson.error("供应商用户id不能为空", null);
        }
        return shopService.updateAllAuthImage(authUserId);
    }

    @ApiOperation("管理员生成供应商token")
    @GetMapping("/token/generate")
    public ResponseJson<String> generateShopToken(Integer authUserId, HttpServletRequest request) {
        // 由管理员调用接口，无法通过@CurrentUser注解获取供应商用户id
        // 对管理员身份进行校验
        String token = request.getHeader("X-Token");
        String cacheToken = null != token ? String.valueOf(redisService.get(token)) : null;
        if (null != cacheToken && JwtUtil.isVerify(cacheToken)) {
            int adminId = JwtUtil.parseTokenUid(cacheToken);
            SysUser user = systemMapper.getUser(adminId);
            if (null == user || 1 != user.getUserIdentity()) {
                return ResponseJson.error("生成token失败", null);
            } else {
                // 生成token给用户
                String shopToken = JwtUtil.createToken(authUserId);
                // 为了过期续签，将token存入redis，并设置超时时间
                redisService.set(shopToken, shopToken, JwtUtil.getExpireTime());
                return ResponseJson.success(shopToken);
            }
        } else {
            return ResponseJson.error("生成token失败", null);
        }
    }
}
