package com.caimei.service.sys.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.caimei.mapper.cmMapper.SystemMapper;
import com.caimei.model.ResponseJson;
import com.caimei.model.po.SysMenu;
import com.caimei.model.po.SysMenuTree;
import com.caimei.model.po.SysPermission;
import com.caimei.model.po.SysRole;
import com.caimei.service.sys.SysMenuService;
import com.caimei.service.sys.SysRoleService;
import com.caimei.utils.CommonUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import static com.alibaba.fastjson.JSON.parseArray;

/**
 * Description
 *
 * @author : Charles
 * @date : 2022/1/6
 */
@Slf4j
@Service
public class SysMenuServiceImpl implements SysMenuService {

    @Resource
    private SystemMapper systemMapper;

    @Resource
    private SysRoleService sysRoleService;

    /**
     * 获取菜单列表
     *  @param parentId 父级菜单Id
     * @param status   状态：0启用，1停用
     * @param pageNum  页码
     * @param pageSize 每页大小
     */
    @Override
    public ResponseJson<PageInfo<SysMenu>> getMenuList(Integer parentId, Integer menuType, Integer status, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<SysMenu> menus = systemMapper.getMenuList(parentId, menuType, status);
        PageInfo<SysMenu> pageData = new PageInfo<>(menus);
        return ResponseJson.success(pageData);
    }

    /**
     * 获取所有启用的菜单列表（树状结构返回）
     */
    @Override
    public ResponseJson<List<SysMenuTree>> getMenuTree(Integer menuType) {
        menuType = null == menuType ? 1 : menuType;
        List<SysMenuTree> menus = systemMapper.getMenuTree("0", menuType);
        // 递归设置子菜单
        setSubMenusData(menus);
        return ResponseJson.success(menus);
    }

    /**
     * 递归设置子菜单（树状结构）
     */
    private void setSubMenusData(List<SysMenuTree> menus) {
        if (!CollectionUtils.isEmpty(menus)) {
            for (SysMenuTree menu : menus) {
                // 获取子菜单
                List<SysMenuTree> submenus = systemMapper.getMenuTree(menu.getId(), null);
                if (!CollectionUtils.isEmpty(submenus)) {
                    setSubMenusData(submenus);
                    menu.setSubMenus(submenus);
                } else {
                    // 获取当前菜单的按钮权限
                    List<SysMenuTree> permissions = systemMapper.getTreePermissions(menu.getId());
                    if (!CollectionUtils.isEmpty(permissions)) {
                        menu.setSubMenus(permissions);
                    }
                }
            }
        }
    }

    /**
     * 根据ID获取菜单
     */
    @Override
    public ResponseJson<SysMenu> getMenu(Integer id) {
        SysMenu menu = systemMapper.getMenu(id);
        // 获取按钮权限
        List<SysPermission> permissions = systemMapper.getPermissionList(menu.getId());
        if (!CollectionUtils.isEmpty(permissions)) {
            menu.setPermissions(permissions);
        }
        return ResponseJson.success(menu);
    }

    /**
     * 根据ID更新菜单
     */
    @Transactional(rollbackFor=Exception.class)
    @Override
    public ResponseJson<Void> updateMenu(Integer id, SysMenu menu) {
        menu.setId(id);
        menu.setDelFlag(0);
        SysMenu dbMenu = systemMapper.getMenuByName(menu.getMenuType(), menu.getName());
        if (null != dbMenu && !dbMenu.getId().equals(id)) {
            return ResponseJson.error("菜单路由名已存在", null);
        }
        systemMapper.updateMenu(menu);
        // 按钮权限JSON数据解析
        ResponseJson<List<SysPermission>> result = parsePermissionJson(menu.getId(), menu.getPermissionJson());
        if (-1 == result.getCode()) {
            return ResponseJson.error(result.getMsg(), null);
        }
        List<SysPermission> permissionList = result.getData();
        Integer baseParentId = getBaseParentId(menu.getId());
        SysMenu baseParentMenu = systemMapper.getMenu(baseParentId);
        menu.setBaseParentId(baseParentId);
        if (2 == menu.getMenuType()) {
            // 移除菜单与角色的关联
            removeMenuRoleRelation(menu, dbMenu);
            if (1 == baseParentMenu.getBaseFlag()) {
                // 供应商基础菜单
                addShopRoleRelation(menu, permissionList);
            } else {
                // 供应商会员菜单
                ResponseJson<Void> saveResult = saveVipMenu(menu, permissionList);
                if (0 != saveResult.getCode()) {
                    return saveResult;
                }
            }
        }
        return ResponseJson.success();
    }

    /**
     * 根据ID(可选择的)更新菜单字段
     */
    @Transactional(rollbackFor=Exception.class)
    @Override
    public ResponseJson<Void> updateMenuSelective(Integer id, SysMenu menu) {
        SysMenu dbMenu = systemMapper.getMenu(id);
        if (StringUtils.hasLength(menu.getTitle())){
            dbMenu.setTitle(menu.getTitle());
        }
        if (StringUtils.hasLength(menu.getName())){
            dbMenu.setName(menu.getName());
        }
        if (StringUtils.hasLength(menu.getIcon())){
            dbMenu.setIcon(menu.getIcon());
        }
        if (null != menu.getParentId()){
            dbMenu.setParentId(menu.getParentId());
        }
        if (null != menu.getSort()){
            dbMenu.setSort(menu.getSort());
        }
        if (null != menu.getHidden()){
            dbMenu.setHidden(menu.getHidden());
        }
        if (null != menu.getStatus()){
            dbMenu.setStatus(menu.getStatus());
        }
        systemMapper.updateMenu(dbMenu);
        // 按钮权限JSON数据解析
        ResponseJson<List<SysPermission>> error = parsePermissionJson(menu.getId(), menu.getPermissionJson());
        if (-1 == error.getCode()) {
            return ResponseJson.error(error.getMsg(), null);
        }
        return ResponseJson.success();
    }

    /**
     * 添加菜单
     */
    @Transactional(rollbackFor=Exception.class)
    @Override
    public ResponseJson<Void> addMenu(SysMenu menu) {
        if (null == menu.getParentId()) {
            menu.setParentId(0);
        }
        if (!StringUtils.hasLength(menu.getName())) {
            return ResponseJson.error("菜单路由名不能为空！", null);
        }
        if (!StringUtils.hasLength(menu.getTitle())) {
            return ResponseJson.error("菜单名称不能为空！", null);
        }
        log.info(menu.getMenuType() + "," + menu.getName());
        // 根据路由名称查询菜单
        SysMenu dbMenu = systemMapper.getMenuByName(menu.getMenuType(), menu.getName());
        if (null != dbMenu) {
            log.info(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>" + dbMenu.getId());
        }
        if (null == dbMenu) {
            systemMapper.insertMenu(menu);
            // 按钮权限JSON数据解析
            Integer menuId = menu.getId();
            ResponseJson<List<SysPermission>> result = parsePermissionJson(menuId, menu.getPermissionJson());
            if (-1 == result.getCode()) {
                return ResponseJson.error(result.getMsg(), null);
            }
            List<SysPermission> permissionList = result.getData();
            if (2 == menu.getMenuType()) {
                Integer baseParentId = getBaseParentId(menuId);
                SysMenu baseParentMenu = systemMapper.getMenu(baseParentId);
                menu.setBaseParentId(baseParentId);
                if (1 == baseParentMenu.getBaseFlag()) {
                    // 会员基础菜单
                    addShopRoleRelation(menu, permissionList);
                } else {
                    // 会员订制菜单
                    ResponseJson<Void> saveResult = saveVipMenu(menu, permissionList);
                    if (0 != saveResult.getCode()) {
                        systemMapper.deleteMenu(menuId);
                        return saveResult;
                    }
                }
            }
        }else {
            return ResponseJson.error("菜单路由名已存在", null);
        }
        return ResponseJson.success();
    }



    /**
     * 获取子菜单顶层母菜单id
     * @param menuId
     * @return
     */
    private Integer getBaseParentId(Integer menuId) {
        Integer parentId = systemMapper.getMenuParentId(menuId);
        if (0 != parentId) {
           return getBaseParentId(parentId);
        } else {
            return menuId;
        }
    }

    @Override
    public ResponseJson<Void> saveVipMenu(SysMenu menu, List<SysPermission> permissionList) {
        if (0 == menu.getParentId()) {
            // 供应商订制顶层菜单
            // 保存会员菜单角色
            SysRole vipMenuRole = new SysRole();
            vipMenuRole.setRoleName(menu.getTitle() + "会员菜单角色");
            vipMenuRole.setRoleDesc(menu.getTitle() + "会员菜单角色");
            vipMenuRole.setMenuIds(menu.getId().toString());
            vipMenuRole.setAuthUserId(1);
            vipMenuRole.setRoleType(2);
            Integer dbVipRoleId = systemMapper.getVipRoleIdByMenuId(menu.getId());
            vipMenuRole.setId(dbVipRoleId);
            vipMenuRole.setDelFlag(0);
            ResponseJson<Void> result = null;
            if (null == dbVipRoleId) {
                // 新增会员角色
                result = sysRoleService.addRole(vipMenuRole);
            } else {
                // 更新会员角色
                systemMapper.updateRole(vipMenuRole);
                result = ResponseJson.success();
            }
            if (0 != result.getCode()) {
                return result;
            }
            permissionList.forEach(permission->{
                systemMapper.insertRolePermissionRelation(vipMenuRole.getId(), permission.getId());
            });
        } else {
            // 供应商订制子菜单
            // 查询顶层父级菜单对应角色id
            Integer baseParentRoleId = systemMapper.getVipRoleIdByMenuId(menu.getBaseParentId());
            // 保存角色菜单关系
            systemMapper.insertRoleMenuRelation(baseParentRoleId, menu.getId());
        }
        return ResponseJson.success();
    }

    private void addShopRoleRelation(SysMenu menu, List<SysPermission> permissionList) {
        Integer menuId = menu.getId();
        // 添加供应商基础菜单时将该菜单与供应商基础角色关联
        // 根据角色ID获取菜单Ids
        List<Integer> menuIdList = systemMapper.getMenuIdsByRoleId(1);
        if (!menuIdList.contains(menuId)) {
            menuIdList.add(menuId);
            SysRole sysRole = new SysRole();
            String menuIds = CommonUtil.idListToString(menuIdList, ",");
            sysRole.setMenuIds(menuIds);
            sysRoleService.updateRole(1, sysRole);
        }
        // 添加权限关联
        permissionList.forEach(permission->{
            systemMapper.insertRolePermissionRelation(1,permission.getId());
        });
    }


    private void removeMenuRoleRelation(SysMenu menu, SysMenu dbMenu) {
        Integer menuId = menu.getId();
        // 根据角色ID获取菜单Ids
        List<Integer> menuIdList = systemMapper.getMenuIdsByRoleId(1);
        if (menuIdList.contains(menuId)) {
            //之前是基础菜单
            menuIdList.remove(menuId);
            String menuIds = CommonUtil.idListToString(menuIdList, ",");
            SysRole sysRole = new SysRole();
            sysRole.setMenuIds(menuIds);
            sysRoleService.updateRole(1, sysRole);
        } else {
            // 之前是订制菜单
            Integer roleId = systemMapper.getVipRoleIdByMenuId(menuId);
            if (1 == menu.getBaseFlag() && 0 == menu.getParentId()) {
                // 只有顶层订制菜单更改为基础菜单才删除原角色
                systemMapper.deleteRole(roleId);
                systemMapper.deleteMenuRoleRelation(menuId);
            } else if (0 != menu.getParentId()) {
                // 子菜单删除菜单角色关联
                systemMapper.deleteMenuRoleRelation(menuId);
            }
        }
        // 根据菜单ID获取权限关联信息
        systemMapper.deleteRoleMenuPermission(menuId);
    }

    /**
     * 根据ID删除菜单
     */
    @Transactional(rollbackFor=Exception.class)
    @Override
    public ResponseJson<Void> deleteMenu(Integer id) {
        // 删除该菜单对应的会员角色和关联
        Integer vipRoleId = systemMapper.getVipRoleIdByMenuId(id);
        SysMenu menu = systemMapper.getMenu(id);
        systemMapper.deleteMenuRoleRelation(id);
        if (0 == menu.getParentId()) {
            systemMapper.deleteRole(vipRoleId);
        }
        List<Integer> permissionIds = systemMapper.getPermissionIds(id);
        // 删除该菜单权限按钮
        for (Integer permissionId : permissionIds) {
           systemMapper.deletePermission(permissionId);
        }
        // 删除菜单
        systemMapper.deleteMenu(id);
        // 获取子菜单列表,递归删除
        List<SysMenu> childMenuList = systemMapper.getMenuList(id, null, null);
        childMenuList.forEach(childMenu-> deleteMenu(childMenu.getId()));
        return ResponseJson.success();
    }

    /**
     * 按钮权限JSON数据解析
     * @param menuId 菜单Id
     * @param jsonStr 按钮权限JSON串
     * @return
     */
    private ResponseJson<List<SysPermission>> parsePermissionJson(Integer menuId, String jsonStr) {
        // 按钮权限JSON数据解析
        JSONArray permissionJson = null;
        List<SysPermission> permissionList = new ArrayList<>();
        if (StringUtils.hasLength(jsonStr) && !"[]".equals(jsonStr)) {
            try {
                permissionJson = parseArray(jsonStr);
                if (null != permissionJson && !permissionJson.isEmpty()) {
                    return ResponseJson.error("按钮权限JSON数据异常！", null);
                }
                for (Object permissionObject : permissionJson) {
                    JSONObject detail = (JSONObject) permissionObject;
                    Integer permissionId = (Integer) detail.get("id");
                    String permissionName = (String) detail.get("name");
                    String permissionTitle = (String) detail.get("title");
                    SysPermission item = new SysPermission();
                    item.setId(permissionId);
                    item.setMenuId(menuId);
                    item.setName(permissionName);
                    item.setTitle(permissionTitle);
                    permissionList.add(item);
                }
            } catch (Exception e) {
                log.error("按钮权限JSON数据解析异常try-catch:", e);
                return ResponseJson.error("按钮权限JSON数据解析异常！", null);
            }
        }
        List<Integer> dbPermissionIds = systemMapper.getPermissionIds(menuId);
        List<Integer> permissionIds = new ArrayList<>();
        for (SysPermission permission : permissionList) {
            if (null != permission.getId() && permission.getId() > 0 && dbPermissionIds.contains(permission.getId())) {
                systemMapper.updatePermission(permission);
                permissionIds.add(permission.getId());
            } else {
                systemMapper.insertPermission(permission);
            }
        }
        for (Integer id : dbPermissionIds) {
            if (!permissionIds.contains(id)){
                systemMapper.deletePermission(id);
            }
        }
        return ResponseJson.success(permissionList);
    }



    /**
     * 获取一颗菜单树下的所有菜单id和权限id
     */
    @Override
    public ResponseJson<Void> getAllBaseMenuIds(SysMenu menu, Set<Integer> menuIdSet, Set<Integer> permissionIdSet) {
        // 解除基础菜单与会员角色的关联
        Integer menuId = menu.getId();
        menuIdSet.add(menuId);
        // 获取菜单下所有权限id
        List<Integer> permissionIdList = systemMapper.getPermissionIds(menuId);
        permissionIdSet.addAll(permissionIdList);
        // 获取子菜单列表
        List<SysMenu> childMenuList = systemMapper.getMenuList(menuId, 2, null);
        // 递归获取树下所有基础子菜单id
        childMenuList.forEach(childMenu -> getAllBaseMenuIds(childMenu, menuIdSet, permissionIdSet));
        return ResponseJson.success();
    }

    /**
     * 获取一颗菜单树下的所有菜单id和权限id
     */
    @Override
    public ResponseJson<Void> getAllVipMenuIds(SysMenu menu, Set<Integer> menuIdSet, Set<Integer> permissionIdSet) {
        Integer menuId = menu.getId();
        menuIdSet.add(menuId);
        // 获取菜单下所有权限id
        List<Integer> permissionIdList = systemMapper.getPermissionIds(menuId);
        permissionIdSet.addAll(permissionIdList);
        // 获取子菜单列表
        List<SysMenu> childMenuList = systemMapper.getMenuList(menuId, 2, null);
        // 递归获取树下所有会员子菜单id
        childMenuList.forEach(childMenu -> getAllVipMenuIds(childMenu, menuIdSet, permissionIdSet));
        return ResponseJson.success();
    }

}
