package com.caimei.service.wechat.impl;

import com.caimei.components.RedisService;
import com.caimei.mapper.cmMapper.AuthMapper;
import com.caimei.mapper.cmMapper.ClubMapper;
import com.caimei.mapper.cmMapper.LoginMapper;
import com.caimei.mapper.cmMapper.ShopMapper;
import com.caimei.model.ResponseJson;
import com.caimei.model.vo.AuthVo;
import com.caimei.model.vo.ShopFormVo;
import com.caimei.model.vo.WxClubUserVo;
import com.caimei.service.wechat.LoginService;
import com.caimei.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Map;
import java.util.UUID;

/**
 * Description
 *
 * @author : Aslee
 * @date : 2021/7/15
 */
@Slf4j
@Service
public class LoginServiceImpl implements LoginService {

    @Resource
    private RedisService redisService;
    @Resource
    private LoginMapper loginMapper;
    @Resource
    private ShopMapper shopMapper;
    @Resource
    private ClubMapper clubMapper;
    @Resource
    private AuthMapper authMapper;

    @Value("${spring.profiles.active}")
    private String active;

    @Override
    public ResponseJson<WxClubUserVo> loginByAuthorization(String code, String appId) {
        if (StringUtils.isEmpty(code)) {
            return ResponseJson.error("参数异常,请输入code", null);
        }
        if (StringUtils.isEmpty(appId)) {
            return ResponseJson.error("参数异常,请输入appId", null);
        }
        // 获取供应商公众号appId对应的appSecret
        String appSecret = shopMapper.getAppSecretByAppId(appId);
        if (StringUtils.isEmpty(appSecret)) {
            return ResponseJson.error("参数异常，该公众号不存在", null);
        }
        Map<String,String> infoMap;
        try {
            infoMap = WxUtils.getAccessToken(code, appId, appSecret);
        } catch (Exception e) {
            log.info("微信服务器异常", e);
            return ResponseJson.error(-1, "微信服务器异常", null);
        }
        if ("-1".equals(infoMap.get("code"))) {
            return ResponseJson.error(infoMap.get("errmsg"), null);
        }
        //解析相应内容（转换成json对象）
        String openId = infoMap.get("openId");
        String accessToken = infoMap.get("accessToken");
        log.info("微信授权，openId:>>>>>>>>>>>" + openId);
        WxClubUserVo clubUser = loginMapper.getClubUserByOpenId(openId, appId);
        if (null == clubUser) {
            clubUser = new WxClubUserVo();
            clubUser.setOpenId(openId);
            clubUser.setAccessToken(accessToken);
            return ResponseJson.error(-2, "未绑定微信", clubUser);
        }
        clubUser.setAccessToken(accessToken);
        // 登录成功redis保存token
        redisService.set(accessToken, openId + "|" + appId, 60L * 60 * 4);
        return ResponseJson.success(clubUser);
    }

    @Override
    public ResponseJson sendVerifyCode(String mobile, Integer authUserId, Integer type) {
        if (StringUtils.isEmpty(mobile)) {
            return ResponseJson.error("参数异常,请输入手机号");
        }
        if (null == authUserId) {
            return ResponseJson.error("参数异常,请输入供应商用户id");
        }
        // 获取供应商名称
        ShopFormVo shop = shopMapper.getShopByUserId(authUserId);
        String shopName = "";
        if (null == shop) {
            return ResponseJson.error("供应商不存在");
        } else {
            shopName = shop.getShopName();
        }
        String verifyCode = "prod".equals(active) ? CodeUtil.generateCodeInt(6) : "666666";
        String content = "欢迎登录" + shopName + "会员，您的短信验证码为：" + verifyCode + "，该验证码 5 分钟内有效，请勿泄漏于他人。";
        Boolean sendSms = SmsUtils.sendSms(1, mobile, content);
        if (!sendSms) {
            log.info("会员登录验证码发送失败，手机号:" + mobile);
            return ResponseJson.error("发送失败,请确认手机号无误");
        } else {
            log.info("会员登录验证码发送成功，手机号:" + mobile + ",验证码：" + verifyCode);
        }
        log.info(content);
        redisService.set("code:" + mobile, verifyCode, 300L);
        return ResponseJson.success("发送成功");
    }


    @Override
    public ResponseJson<WxClubUserVo> loginByVerifyCode(String mobile, String verifyCode, String accessToken, String openId, String appId) throws Exception {
        if (StringUtils.isEmpty(mobile)) {
            return ResponseJson.error("参数异常，请输入手机号", null);
        }
        if (StringUtils.isEmpty(verifyCode)) {
            return ResponseJson.error("参数异常，请输入验证码", null);
        }
        if (StringUtils.isEmpty(accessToken)) {
            return ResponseJson.error("参数异常，请输入accessToken", null);
        }
        if (StringUtils.isEmpty(openId)) {
            return ResponseJson.error("参数异常，请输入openId", null);
        }
        if (StringUtils.isEmpty(appId)) {
            return ResponseJson.error("参数异常，请输入appId", null);
        }
        // 校验验证码是否正确
        String redisVerifyCode = null == redisService.get("code:" + mobile) ? null : redisService.get("code:" + mobile).toString();
        if (!verifyCode.equals(redisVerifyCode)) {
            return ResponseJson.error("验证码错误，请重新输入", null);
        }
        // 校验机构用户是否存在
        WxClubUserVo wxClubUser = clubMapper.getWxClubUser(mobile, appId, null, null, null);
        if (null == wxClubUser || 1 != wxClubUser.getStatus()) {
            return ResponseJson.error("抱歉，您的手机号暂无权限登录", null);
        }
        // 校验机构和供应商的状态是否为已上线
        AuthVo auth = authMapper.getAuthById(wxClubUser.getAuthId());
        Integer shopStatus = shopMapper.getShopStatus(auth.getAuthUserId());
        if (1 != auth.getStatus() || 1 != shopStatus) {
            return ResponseJson.error("登录失败", null);
        }
        // 校验appId是否和机构所属供应商的appId相同
        String dbAppId = shopMapper.getAppId(auth.getAuthUserId());
        if (StringUtils.isEmpty(dbAppId) || !appId.equals(dbAppId)) {
            return ResponseJson.error("登录失败", null);
        }
        Map<String, String> userInfo = WxUtils.getUserInfo(accessToken, openId);
        String nickName = userInfo.get("nickName");
        loginMapper.bindClubUser(mobile, nickName, openId);
        WxClubUserVo clubUser = new WxClubUserVo();
        clubUser.setClubUserId(wxClubUser.getClubUserId());
        clubUser.setAuthId(wxClubUser.getAuthId());
        clubUser.setAuthUserId(auth.getAuthUserId());
        clubUser.setMobile(mobile);
        clubUser.setOpenId(openId);
        clubUser.setAccessToken(accessToken);
        // 登录成功redis保存token
        redisService.set(accessToken, openId + "|" + appId, 60L * 60 * 4);
        return ResponseJson.success(clubUser);
    }

    @Override
    public ResponseJson<WxClubUserVo> loginByVerifyCode(String mobile, String verifyCode, Integer authUserId) {
        if (StringUtils.isEmpty(mobile)) {
            return ResponseJson.error("参数异常，请输入手机号", null);
        }
        if (StringUtils.isEmpty(verifyCode)) {
            return ResponseJson.error("参数异常，请输入验证码", null);
        }
        if (null == authUserId) {
            return ResponseJson.error("参数异常，请输入供应商用户id", null);
        }
        // 校验验证码是否正确
        String redisVerifyCode = null == redisService.get("code:" + mobile) ? null : redisService.get("code:" + mobile).toString();
        if (!verifyCode.equals(redisVerifyCode)) {
            return ResponseJson.error("验证码错误，请重新输入", null);
        }
        // 获取手机号对应机构用户
        WxClubUserVo clubUser = clubMapper.getWxClubUser(mobile, null, authUserId, null, null);
        if (null == clubUser || 1 != clubUser.getStatus()) {
            clubUser = new WxClubUserVo();
            clubUser.setMobile(mobile);
            return ResponseJson.error("抱歉，您的手机号暂无权限登录", clubUser);
        }
        // 校验机构和供应商的状态是否为已上线
        // AuthVo auth = authMapper.getAuthById(clubUser.getAuthId());
        /*Integer shopStatus = shopMapper.getShopStatus(clubUser.getAuthUserId());
        if (1 != auth.getStatus() || 1 != shopStatus) {
            return ResponseJson.error("登录失败", null);
        }*/
        // 登录成功redis保存token
        String accessToken = UUID.randomUUID().toString();
        redisService.set(accessToken, mobile + "," + authUserId, 60L * 60 * 24);
        clubUser.setAccessToken(accessToken);
        return ResponseJson.success(clubUser);
    }

    @Override
    public ResponseJson<WxClubUserVo> passwordLogin(String mobile, String password, Integer authUserId) {
        // 获取手机号对应机构用户
        WxClubUserVo clubUser = clubMapper.getWxClubUser(mobile, null, authUserId, null, null);
        Boolean passwordVerify;
        if (null == clubUser) {
            return ResponseJson.error("用户不存在", null);
        } else if (1 != clubUser.getStatus()) {
            return ResponseJson.error("账号被停用", null);
        } else {
            String dbPassword = clubMapper.getDbPassword(clubUser.getClubUserId());
            // 比对密码
            passwordVerify = passwordVerify(password, dbPassword);
            if (passwordVerify) {
                // 登录成功redis保存token
                String accessToken = UUID.randomUUID().toString();
                // +","+clubUser.getClubUserId()
                redisService.set(accessToken, mobile + "," + authUserId + "," + clubUser.getClubUserId(), 60L * 60 * 24);
                clubUser.setAccessToken(accessToken);
                return ResponseJson.success(clubUser);
            }
        }
        return ResponseJson.error("输入的密码和账户名不匹配", null);
    }
    @Override
    public ResponseJson<WxClubUserVo> getUserInfoByToken(HttpServletRequest request) {
        //获取redis存储的token
        String tokenStr = request.getHeader("X-Token");
        String token = redisService.get(tokenStr).toString();
        if(JwtUtil.isVerify(token)){
            return ResponseJson.error("token不存在", null);
        }
        String[] club = token.split(",");
        WxClubUserVo userInfoByToken = clubMapper.getUserInfoByToken(Integer.valueOf(club[2]));
        userInfoByToken.setAccessToken(tokenStr);
        //从token中获取clubuserid
        return ResponseJson.success(userInfoByToken);
    }

        /**
         * 密码校验
         */
    private Boolean passwordVerify(String password, String dbPassword) {
        String md5Password = Md5Util.md5(password);
        return md5Password.equals(dbPassword);
    }
}
